﻿using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopContracts.ViewModels;
using IceCreamShopDataModels.Models;
using Microsoft.Extensions.Logging;

namespace IceCreamShopBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;

        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. ShopName: {ShopName}. Id: {Id}", model?.ShopName, model?.Id);
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement.  ShopName: {ShopName}. Id: {Id}", model.ShopName, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id: {Id}", element.Id);
            return element;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public bool MakeShipment(ShopSearchModel shopModel, IIceCreamModel iceCream, int count)
        {
            if (shopModel == null)
            {
                throw new ArgumentNullException(nameof(shopModel));
            }
            if (iceCream == null)
            {
                throw new ArgumentNullException(nameof(iceCream));
            }
            if (count <= 0)
            {
                throw new ArgumentException("Количество товаров(мороженого) в магазине должно быть больше нуля", nameof(count));
            }
            _logger.LogInformation("MakeShipment(GetElement). ShopName: {ShopName}. Id: {Id}", shopModel.ShopName, shopModel.Id);
            var shop = _shopStorage.GetElement(shopModel);
            if (shop == null)
            {
                _logger.LogWarning("MakeShipment(GetElement). Element not found");
                return false;
            }
            if (shop.IceCreamsMaximum - shop.ShopIceCreams.Sum(x => x.Value.Item2) < count)
            {
                _logger.LogWarning("MakeShipment error. No space for new ice creams");
                return false;
            }
            if (shop.ShopIceCreams.ContainsKey(iceCream.Id))
            {
                var shopIC = shop.ShopIceCreams[iceCream.Id];
                shopIC.Item2 += count;
                shop.ShopIceCreams[iceCream.Id] = shopIC;
                _logger.LogInformation("MakeShipment. Added {count} '{iceCream}' to '{ShopName}' shop", count, iceCream.IceCreamName, 
                    shop.ShopName);
            } 
            else
            {
                shop.ShopIceCreams.Add(iceCream.Id, (iceCream, count));
                _logger.LogInformation("MakeShipment. Added {count} new '{iceCream}' to '{ShopName}' shop", count, iceCream.IceCreamName, 
                    shop.ShopName);
            }
            if (_shopStorage.Update(new ShopBindingModel()
            {
                Id = shop.Id,
                ShopName = shop.ShopName,
                Address = shop.Address,
                DateOpening = shop.DateOpening,
                IceCreamsMaximum = shop.IceCreamsMaximum,
                ShopIceCreams = shop.ShopIceCreams,
            }) == null)
            {
                _logger.LogWarning("MakeShipment. Update operation failed");
                return false;
            }
            return true;
        }

        public bool MakeSale(IIceCreamModel model, int count)
        {
            return _shopStorage.MakeSale(model, count);
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
            }
            if (string.IsNullOrEmpty(model.Address))
            {
                throw new ArgumentNullException("Нет адреса магазина", nameof(model.Address));
            }
            _logger.LogInformation("Shop. ShopName: {ShopName}. Address: {Address}. Id: {Id}", model.ShopName, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                ShopName = model.ShopName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
